"use strict";
var binding_scope_1 = require("../bindings/binding_scope");
var Resolver = (function () {
    function Resolver(middleWare) {
        if (middleWare === void 0) { middleWare = []; }
        this._middleWare = middleWare;
    }
    Resolver.prototype.resolve = function (context) {
        var rootRequest = context.plan.rootRequest;
        return this._construct(rootRequest);
    };
    Resolver.prototype._construct = function (request) {
        var _this = this;
        var childRequests = request.childRequests;
        var binding = request.bindings[0];
        var constr = binding.implementationType;
        var isSingleton = binding.scope === binding_scope_1.default.Singleton;
        if (isSingleton && binding.cache !== null) {
            return binding.cache;
        }
        if (childRequests.length > 0) {
            var injections = childRequests.map(function (childRequest) {
                return _this._construct(childRequest);
            });
            var instance = this._createInstance(constr, injections);
            if (isSingleton) {
                binding.cache = instance;
            }
            return instance;
        }
        else {
            var instance = new constr();
            if (isSingleton) {
                binding.cache = instance;
            }
            return instance;
        }
    };
    Resolver.prototype._createInstance = function (Func, injections) {
        return new (Func.bind.apply(Func, [void 0].concat(injections)))();
    };
    return Resolver;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Resolver;
